/*
 * Copyright (c) 2010 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5r2.
 * 
 * SpaceNet 2.5r2 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5r2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5r2.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.domain.network.edge;

import java.util.ArrayList;
import java.util.List;

import edu.mit.spacenet.util.GlobalParameters;

/**
 * Edge that represents a series of propulsive burns with a finite duration.
 * 
 * @author Paul Grogan
 */
public class SpaceEdge extends Edge {
	private double duration;
	private List<Burn> burns;
	
	/**
	 * The default constructor that initializes the list of burns.
	 */
	public SpaceEdge() { 
		super(); 
		burns = new ArrayList<Burn>();
	}
	
	/**
	 * Gets the duration.
	 * 
	 * @return the duration (days)
	 */
	public double getDuration() {
		return GlobalParameters.getRoundedTime(duration);
	}
	
	/**
	 * Sets the duration, rounding to nearest time precision.
	 * 
	 * @param duration the duration (days)
	 */
	public void setDuration(double duration) {
		this.duration = duration;
	}
	
	/**
	 * Gets the list of burns.
	 * 
	 * @return the list of burns
	 */
	public List<Burn> getBurns() {
		return burns;
	}
	
	/**
	 * Sets the list of burns.
	 * 
	 * @param burns the list of burns
	 */
	public void setBurns(List<Burn> burns) {
		this.burns = burns;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.network.edge.Edge#getEdgeType()
	 */
	@Override
	public EdgeType getEdgeType() {
		return EdgeType.SPACE;
	}
}
